%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%               Integrate
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
lambda = 1;

if (1/2*p_vec_o*p_vec_o' - 1/sqrt(q_vec_o*q_vec_o') + ...
        Hp(q_vec_o, p_vec_o, epsilon1, epsilon2, epsilon3, epsilon4, epsilon5, Hp1, Hp2, Hp3, Hp4, Hp5)) >= 0
    q_str = num2str(q_vec_o);
    p_str = num2str(p_vec_o);
    warndlg(sprintf(['With these initial conditions\n',...
        '       q =  ', q_str,'\n',...
        '       p =  ', p_str,'\n',...
        'the total energy is positive or null. ',...
        'The Regularization method is not able to handle this case. ',...
        'Try with ''Integrate'' of the Standard method to compute the orbit.']))
    return
end

lambda = 1/sqrt(-2*(1/2*p_vec_o*p_vec_o' - 1/sqrt(q_vec_o*q_vec_o')...
    + Hp(q_vec_o, p_vec_o, epsilon1, epsilon2, epsilon3, epsilon4, epsilon5, Hp1, Hp2, Hp3, Hp4, Hp5)));
%--------------------------------------------------------------------------
er1 = abs(epsilon1*lambda^(2*deg1+2));
er2 = abs(epsilon2*lambda^(2*deg2+2));
er3 = abs(epsilon3*lambda^(2*deg3+2));
er4 = abs(epsilon4*lambda^(2*deg4+2));
er5 = abs(epsilon5*lambda^(2*deg5+2));
if (er1 > 0.2 | er2 > 0.2 | er3 > 0.2 | er4 > 0.2 | er5 > 0.2) &...
        strcmp(get(handles.warningbox, 'Checked'),'on') &...
        (get(handles.radiobutton_Pert,'Value') == 1)
    if strcmp(Hp1,'Zero')
        er1 = '---';
    else
        er1 = num2str(er1);
    end
    if strcmp(Hp2,'Zero')
        er2 = '---';
    else
        er2 = num2str(er2);
    end
    if strcmp(Hp3,'Zero')
        er3 = '---';
    else
        er3 = num2str(er3);
    end
    if strcmp(Hp4,'Zero')
        er4 = '---';
    else
        er4 = num2str(er4);
    end
    if strcmp(Hp5,'Zero')
        er5 = '---';
    else
        er5 = num2str(er5);
    end
    button = questdlg(sprintf(['The absolute values of the rescaled perturbative parameters are, respectively\n',...
        '                    ',er1,'   ',er2,'   ',er3,'    ',er4,'    ',er5,'.\n',...
        'One (or more) of these values is greater than 0.2, so the hypothesis of\n',...
        'small perturbation is not satisfied and the numerical result may be not reliable.\n',...
        'Look at ''Integration Errors'' and compare with the similar outputs of the \n',...
        '''Non perturbative method'' and of the ''Standard method''.\n',...
        'Do you continue anyway?']),...
        'Too large perturbation', 'No');
    if strcmp(button,'Yes')
    else
        return
    end
end
%--------------------------------------------------------------------------
x_vec_o = 1/(lambda^2)*q_vec_o;
y_vec_o = lambda*p_vec_o;
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
if isdeployed
    if get(handles.radiobutton_Pert,'Value') == 1
        [Uo, Vo] = xy2uv(x_vec_o, y_vec_o);
    elseif get(handles.radiobutton_NonPert,'Value') == 1
        [z_vec_o,  w_vec_o] = xy2zw(x_vec_o,  y_vec_o);
        Uo = z_vec_o';
        Vo = w_vec_o';        
    end 
else
    if get(handles.radiobutton_Pert,'Value') == 1
        Symbolic
        Write2file
        waitfor(exist('DiffEquation'))
        [Uo, Vo] = xy2uv(x_vec_o, y_vec_o);
    elseif get(handles.radiobutton_NonPert,'Value') == 1
        SymbolicNonPert
        Write2fileNonPert
        waitfor(exist('DiffEquation'))
        [z_vec_o,  w_vec_o] = xy2zw(x_vec_o,  y_vec_o);
        Uo = z_vec_o';
        Vo = w_vec_o';        
    end
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

block = str2num(get(handles.edit_block,'String'));
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                     Tom
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
if strcmp(ODE, 'Tom') == 1
    met = str2num(get(handles.edit_met,'String'));
    StepSize = str2num(get(handles.edit_StepSize,'String'));
    TomOutputStep = str2num(get(handles.edit_StepSize,'String'));
    RelTolNewt = str2num(get(handles.edit_RelTolNewt,'String'));
    Itnlmax = str2num(get(handles.edit_Itnlmax,'String'));

    if strcmp(get(handles.waitbars, 'Checked'),'on')
        global pick                                              %%%%%     waitbar
        h = waitbar(0,'Please wait..., KEPLER is calculating.'); %%%%%     waitbar
        pick = -tau_total/100;                                   %%%%%     waitbar
    end

    if strcmp(get(handles.SlowWaitbars, 'Checked'),'on')
        begin = datestr(now, 'dd mmm HH:MM');
        slow = waitbar(0,'Please wait..., KEPLER is calculating.'); %%%%%     slow waitbar
    end    

    tom_options = tomgset('met',met,'Order',10,'StepSize',StepSize,...
        'RelTolNewt',RelTolNewt,...
        'Vectorized','off','OutputSteps',ceil(TomOutputStep/StepSize),...
        'Itnlmax',Itnlmax,'Stats_step','off','Stats','off');
    tic

    if block
%--------------------------------------------------------------------------
        longblock = str2num(get(handles.edit_ClearMemory,'String'));
        deltaFMFT = 2^(floor(log2(str2num(get(handles.ComputeLFA,'String'))/TomOutputStep)));
        n_cycle = floor(tau_total/block); totslow = floor(tau_total/longblock);
        k = 1; slowcount = 1; w = []; tau_ = [];
        for k = 1:n_cycle
            if length(w) >= longblock/TomOutputStep - 1;
                if (strcmp(get(handles.LGG3_long_FMFT, 'Checked'),'on')) |...
                        (strcmp(get(handles.LSD_long_FMFT, 'Checked'),'on'))  
                    Long_Freq_Analysis
                end
                clear w tau_
                w = []; tau_ = [];
                if strcmp(get(handles.SlowWaitbars, 'Checked'),'on')
                    update = ['Please, wait: ', num2str(floor(slowcount/totslow*100)), '% completed. Starting time: ', begin];
                    waitbar(slowcount/totslow, slow, update);
                end
                slowcount = slowcount + 1;
            end            
            sol = tomgm('DiffEquation', [(k-1)*block  k*block], [Uo; Vo], tom_options);
            W = sol.y';
            tau_c = sol.x;    
            w = [w; W(1:length(W)-1, :)];
            tau_ = [tau_; tau_c(1:length(W)-1)];
            W_o = W(length(W), :);
            Uo = W_o(1:4)';
            Vo = W_o(5:8)';

            if get(handles.radiobutton_Pert,'Value') == 1
                [xo yo] = uv2xy(Uo, Vo);       
                [Uo Vo] = xy2uv(xo, yo);        
                [xo yo] = uv2xy(Uo, Vo);
            elseif get(handles.radiobutton_NonPert,'Value') == 1
                [xo yo] = zw2xy(Uo, Vo);
                [Uo Vo] = xy2zw(xo, yo);           
                [xo yo] = zw2xy(Uo, Vo);
            end

%---------------------- SOSTITUZIONE ---------------------------------------------------------                
%{
                Ktot = 1.001;
                while abs(Ktot-1) > 5*eps
                    Ktot = 1/2*sqrt(xo*xo')*(yo*yo'+1) + sqrt(xo*xo')*...
                    Kp(xo,yo, lambda, epsilon1, epsilon2, epsilon3, epsilon4,...
                        deg1, deg2, deg3, deg4, Hp1, Hp2, Hp3, Hp4);
                    xo = xo/Ktot;
                end           
%}
                f = @(z)1/2*sqrt(z^2*xo*xo')*(yo*yo'+1) + sqrt(z^2*xo*xo')*...
                    Kp(z*xo,yo, lambda, epsilon1, epsilon2, epsilon3, epsilon4, epsilon5,...
                        deg1, deg2, deg3, deg4, deg5, Hp1, Hp2, Hp3, Hp4, Hp5) - 1;
                a = fzero(f,1);
                xo =a*xo;
%---------------------------------------------------------------------------------------------      

            if get(handles.radiobutton_Pert,'Value') == 1
                [Uo Vo] = xy2uv(xo, yo);
            elseif get(handles.radiobutton_NonPert,'Value') == 1
                [Uo Vo] = xy2zw(xo, yo);
                Uo = Uo'; Vo = Vo';
            end     
        end
        if (n_cycle*block ~= tau_total)
            sol = tomgm('DiffEquation', [n_cycle*block  tau_total], [Uo; Vo], tom_options);
            W = sol.y';
            tau_c = sol.x;
            W = [w; W];
            tau_c = [tau_; tau_c];       
        else
            W = w;
            tau_c = tau_;
        end
%--------------------------------------------------------------------------
    else
        longblock = str2num(get(handles.edit_ClearMemory,'String'));
        deltaFMFT = 2^(floor(log2(str2num(get(handles.ComputeLFA,'String'))/TomOutputStep)));
        n_cycle = floor(tau_total/longblock); totslow = floor(tau_total/longblock);
        k = 1; slowcount = 1; w = []; tau_ = [];
        for k = 1:n_cycle
            if length(w) >= longblock/TomOutputStep - 1;
                if (strcmp(get(handles.LGG3_long_FMFT, 'Checked'),'on')) |...
                        (strcmp(get(handles.LSD_long_FMFT, 'Checked'),'on'))  
                    Long_Freq_Analysis
                end
                clear w tau_
                w = []; tau_ = [];
                if strcmp(get(handles.SlowWaitbars, 'Checked'),'on')
                    update = ['Please, wait: ', num2str(floor(slowcount/totslow*100)), '% completed. Starting time: ', begin];
                    waitbar(slowcount/totslow, slow, update);
                end
                slowcount = slowcount + 1;
            end            
            sol = tomgm('DiffEquation', [(k-1)*longblock  k*longblock], [Uo; Vo], tom_options);
            W = sol.y';
            tau_c = sol.x;    
            w = [w; W(1:length(W)-1, :)];
            tau_ = [tau_; tau_c(1:length(W)-1)];
            W_o = W(length(W), :);
            Uo = W_o(1:4)';
            Vo = W_o(5:8)';
        end
        if (n_cycle*longblock ~= tau_total)
            sol = tomgm('DiffEquation', [n_cycle*longblock  tau_total], [Uo; Vo], tom_options);
            W = sol.y';
            tau_c = sol.x;
            W = [w; W];
            tau_c = [tau_; tau_c];       
        else
            W = w;
            tau_c = tau_;
        end
    end
    t_elaps = toc;
    t_elaps = num2str(t_elaps);    
    if strcmp(get(handles.waitbars, 'Checked'),'on')
        close(h)                                                 %%%%%     waitbar
    end

    if strcmp(get(handles.SlowWaitbars, 'Checked'),'on')
        waitbar(1, slow, 'Finish!!!')
%       close(slow)                                             %%%%%     slow waitbar
    end         

    if strcmp(get(handles.perform, 'Checked'),'on')
        if strcmp(num2str(perfid), '')
            perfid = fopen( 'Perform/Perform.txt', 'wt');
            fprintf(perfid, '*************************************************************************************************\n');
        end
        fprintf(perfid, '                             Regularization method          \n\n');
        fprintf(perfid, ['ODE solver Tom, Cycle number ',num2str(tau_total),...
            ', Step size: ',num2str(StepSize),...
            ', Integration time ',t_elaps,' sec.\n']);
        fprintf(perfid, '*************************************************************************************************\n');
    end
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                     Runge-Kutta
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
elseif (strcmp(ODE, 'RungeKutta4') == 1) | (strcmp(ODE, 'RungeKutta5') == 1)
    StepSize = str2num(get(handles.edit_FixedStepSize,'String'));  
    if strcmp(get(handles.waitbars, 'Checked'),'on')
        global pick                                              %%%%%     waitbar
        h = waitbar(0,'Please wait..., KEPLER is calculating.'); %%%%%     waitbar
        pick = -tau_total/100;                                   %%%%%     waitbar
    end

    tau_c = (0 : StepSize : tau_total)';
    tic
    W = eval([ODE, '(@DiffEquation, tau_c, [Uo; Vo])']);
    t_elaps = toc;
    t_elaps = num2str(t_elaps);    
    if strcmp(get(handles.waitbars, 'Checked'),'on')
        close(h)                                                 %%%%%     waitbar
    end

    if strcmp(get(handles.perform, 'Checked'),'on')
        if strcmp(num2str(perfid), '')
            perfid = fopen( 'Perform/Perform.txt', 'wt');
            fprintf(perfid, '*************************************************************************************************\n');
        end
        fprintf(perfid, '                             Regularization method          \n\n');
        fprintf(perfid, ['ODE solver: ',ODE,', Cycle number: ',num2str(tau_total),...
            ', Step size: ',num2str(StepSize),', Integration time: ',t_elaps,' sec.\n']);
        fprintf(perfid, '*************************************************************************************************\n');
    end
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                    IRK-Gauss
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
elseif (strcmp(ODE, 'IRKGauss') == 1)
    StepSize = str2num(get(handles.edit_FixedStepSize,'String'));
    opt.method = str2num(get(handles.edit_IRKMeth,'String'));

    if strcmp(get(handles.waitbars, 'Checked'),'on')
        global pick                                              %%%%%     waitbar
        h = waitbar(0,'Please wait..., KEPLER is calculating.'); %%%%%     waitbar
        pick = -tau_total/100;                                   %%%%%     waitbar
    end

    if strcmp(get(handles.SlowWaitbars, 'Checked'),'on')
        begin = datestr(now, 'dd mmm HH:MM');
        slow = waitbar(0,'Please wait..., KEPLER is calculating.'); %%%%%     slow waitbar
    end    

    tic
    if block
%--------------------------------------------------------------------------
        opt.nstep = block/StepSize;
        longblock = str2num(get(handles.edit_ClearMemory,'String'));
        deltaFMFT = 2^(floor(log2(str2num(get(handles.ComputeLFA,'String'))/StepSize)));
        n_cycle = floor(tau_total/block); totslow = floor(tau_total/longblock);
        k = 1; slowcount = 1; w = []; tau_ = [];
        for k = 1:n_cycle
            if length(w) >= longblock/StepSize - 1;
                if (strcmp(get(handles.LGG3_long_FMFT, 'Checked'),'on')) |...
                        (strcmp(get(handles.LSD_long_FMFT, 'Checked'),'on'))  
                    Long_Freq_Analysis
                end
                clear w tau_
                w = []; tau_ = [];
                if strcmp(get(handles.SlowWaitbars, 'Checked'),'on')
                    update = ['Please, wait: ', num2str(floor(slowcount/totslow*100)), '% completed. Starting time: ', begin];
                    waitbar(slowcount/totslow, slow, update);
                end
                slowcount = slowcount + 1;
            end
            [tau_c,W] = eval([ODE, '(@DiffEquation, [(k-1)*block , k*block], [Uo; Vo], opt)']);
            w = [w; W(1:length(W)-1, :)];
            tau_ = [tau_; tau_c(1:length(W)-1)];
            W_o = W(length(W), :);
            Uo = W_o(1:4)';
            Vo = W_o(5:8)';

            if get(handles.radiobutton_Pert,'Value') == 1
                [xo yo] = uv2xy(Uo, Vo);       
                [Uo Vo] = xy2uv(xo, yo);        
                [xo yo] = uv2xy(Uo, Vo);
            elseif get(handles.radiobutton_NonPert,'Value') == 1
                [xo yo] = zw2xy(Uo, Vo);
                [Uo Vo] = xy2zw(xo, yo);           
                [xo yo] = zw2xy(Uo, Vo);
            end

%---------------------- SOSTITUZIONE ---------------------------------------------------------                
%{
                Ktot = 1.001;
                while abs(Ktot-1) > 5*eps
                    Ktot = 1/2*sqrt(xo*xo')*(yo*yo'+1) + sqrt(xo*xo')*...
                    Kp(xo,yo, lambda, epsilon1, epsilon2, epsilon3, epsilon4,...
                        deg1, deg2, deg3, deg4, Hp1, Hp2, Hp3, Hp4);
                    xo = xo/Ktot;
                end           
%}
                f = @(z)1/2*sqrt(z^2*xo*xo')*(yo*yo'+1) + sqrt(z^2*xo*xo')*...
                    Kp(z*xo,yo, lambda, epsilon1, epsilon2, epsilon3, epsilon4, epsilon5,...
                        deg1, deg2, deg3, deg4, deg5, Hp1, Hp2, Hp3, Hp4, Hp5) - 1;
                a = fzero(f,1);
                xo =a*xo;
%---------------------------------------------------------------------------------------------            

            if get(handles.radiobutton_Pert,'Value') == 1
                [Uo Vo] = xy2uv(xo, yo);
            elseif get(handles.radiobutton_NonPert,'Value') == 1
                [Uo Vo] = xy2zw(xo, yo);
                Uo = Uo'; Vo = Vo';
            end     
        end
        if (n_cycle*block ~= tau_total)
            opt.nstep = (tau_total - n_cycle*block)/StepSize;
            [tau_c,W] = eval([ODE, '(@DiffEquation, [n_cycle*block , tau_total], [Uo; Vo], opt)']);
            W = [w; W];
            tau_c = [tau_; tau_c];
        else
            W = w;
            tau_c = tau_;
        end
%--------------------------------------------------------------------------
    else      
        longblock = str2num(get(handles.edit_ClearMemory,'String'));
        opt.nstep = longblock/StepSize;
        deltaFMFT = 2^(floor(log2(str2num(get(handles.ComputeLFA,'String'))/StepSize)));
        n_cycle = floor(tau_total/longblock); totslow = floor(tau_total/longblock);
        k = 1; slowcount = 1; w = []; tau_ = [];
        for k = 1:n_cycle
            if length(w) >= longblock/StepSize - 1;
                if (strcmp(get(handles.LGG3_long_FMFT, 'Checked'),'on')) |...
                        (strcmp(get(handles.LSD_long_FMFT, 'Checked'),'on'))  
                    Long_Freq_Analysis
                end
                clear w tau_
                w = []; tau_ = [];
                if strcmp(get(handles.SlowWaitbars, 'Checked'),'on')
                    update = ['Please, wait: ', num2str(floor(slowcount/totslow*100)), '% completed. Starting time: ', begin];
                    waitbar(slowcount/totslow, slow, update);
                end
                slowcount = slowcount + 1;
            end
            [tau_c,W] = eval([ODE, '(@DiffEquation, [(k-1)*longblock , k*longblock], [Uo; Vo], opt)']);
            w = [w; W(1:length(W)-1, :)];
            tau_ = [tau_; tau_c(1:length(W)-1)];
            W_o = W(length(W), :);
            Uo = W_o(1:4)';
            Vo = W_o(5:8)';

        end
        if (n_cycle*longblock ~= tau_total)
            opt.nstep = (tau_total - n_cycle*longblock)/StepSize;
            [tau_c,W] = eval([ODE, '(@DiffEquation, [n_cycle*longblock , tau_total], [Uo; Vo], opt)']);
            W = [w; W];
            tau_c = [tau_; tau_c];
        else
            W = w;
            tau_c = tau_;
        end
    end

    t_elaps = toc;
    t_elaps = num2str(t_elaps);    
    if strcmp(get(handles.waitbars, 'Checked'),'on')
        close(h)                                                 %%%%%     waitbar
    end  
    if strcmp(get(handles.SlowWaitbars, 'Checked'),'on')
        waitbar(1, slow, 'Finish!!!')
%       close(slow)                                             %%%%%     slow waitbar
    end         


    if strcmp(get(handles.perform, 'Checked'),'on')
        if strcmp(num2str(perfid), '')
            perfid = fopen( 'Perform/Perform.txt', 'wt');
            fprintf(perfid, '*************************************************************************************************\n');
        end
        fprintf(perfid, '                             Regularization method          \n\n');
        fprintf(perfid, ['ODE solver: ',ODE,', Cycle number: ',num2str(tau_total),...
            ', Step size: ',num2str(StepSize),', Integration time: ',t_elaps,' sec.\n']);
        fprintf(perfid, '*************************************************************************************************\n');
    end       
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                Other ODE solvers
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
else
    if ((strcmp(ODE, 'Odex') == 1) & strcmp(get(handles.singularity, 'Checked'),'on'))
        warndlg({'When using Odex, the ''Watch falling into singularities'' in Window menu must be disabled.';'Be sure that the problem is regular.'})
        return
    end
    Rel_Tol = str2num(get(handles.edit_RelTol,'String'));
    Abs_Tol = str2num(get(handles.edit_AbsTol,'String'));
    Init_Step = str2num(get(handles.edit_InitialStep,'String'));
    Max_Step = str2num(get(handles.edit_MaxStep,'String'));
    OutStep = get(handles.edit_OutputStep,'String');

    if isempty(OutStep)
%----------------------------------------------------------------
%----------------------------------------------------------------
        if (strcmp(get(handles.LGG3_long_FMFT, 'Checked'),'on')) |...
                    (strcmp(get(handles.LSD_long_FMFT, 'Checked'),'on'))  
            warndlg(['You have checked ''Longtime Frequency analysis''.  ',...
               'For a correct Frequency analysis the integration MUST be performed ',...
               'with a constant Output Step (default value: 0.01).  Please, redo ''Integrate''.'])
            set(handles.edit_OutputStep,'String', num2str(0.01));
            set(handles.edit_block,'Visible', 'on')
            set(handles.text_block,'Visible', 'on') 
            return
        end

        if strcmp(get(handles.waitbars, 'Checked'),'on')
            global pick                                              %%%%%     waitbar
            h = waitbar(0,'Please wait..., KEPLER is calculating.'); %%%%%     waitbar
            pick = -tau_total/100;                                   %%%%%     waitbar
        end

        options = odeset('InitialStep',[Init_Step], 'Maxstep',[Max_Step],...
            'RelTol',[Rel_Tol], 'AbsTol',[Abs_Tol]);
        tic             
        [tau_c,W] = eval([ODE, '(''DiffEquation'', [0  tau_total], [Uo; Vo], options)']);
        t_elaps = toc;
        t_elaps = num2str(t_elaps);    
        if strcmp(get(handles.waitbars, 'Checked'),'on')
            close(h)                                                 %%%%%     waitbar
        end
%----------------------------------------------------------------
%----------------------------------------------------------------
    else
        Output_Step = [':',OutStep,':'];

        if strcmp(get(handles.waitbars, 'Checked'),'on')
            global pick                                              %%%%%     waitbar
            h = waitbar(0,'Please wait..., KEPLER is calculating.'); %%%%%     waitbar
            pick = -tau_total/100;                                   %%%%%     waitbar
        end

        if strcmp(get(handles.SlowWaitbars, 'Checked'),'on')
            begin = datestr(now, 'dd mmm HH:MM');
            slow = waitbar(0,'Please wait..., KEPLER is calculating.'); %%%%%     slow waitbar
        end    

        options = odeset('InitialStep',[Init_Step], 'Maxstep',[Max_Step],...
            'RelTol',[Rel_Tol], 'AbsTol',[Abs_Tol]);
        tic             

        if block
%--------------------------------------------------------------------------
            longblock = str2double(get(handles.edit_ClearMemory,'String'));

            if isnan(longblock)
                set(handles.edit_ClearMemory, 'String', 20000);
            end

            deltaFMFT = 2^(floor(log2(str2num(get(handles.ComputeLFA,'String'))/str2num(OutStep))));
            n_cycle = floor(tau_total/block); totslow = floor(tau_total/longblock);
            k = 1; slowcount = 1; w = []; tau_ = [];
            for k = 1:n_cycle
                if length(w) >= longblock/str2num(OutStep) - 1;
                    if (strcmp(get(handles.LGG3_long_FMFT, 'Checked'),'on')) |...
                            (strcmp(get(handles.LSD_long_FMFT, 'Checked'),'on'))  
                        Long_Freq_Analysis
                    end
                    clear w tau_
                    w = []; tau_ = [];
                    if strcmp(get(handles.SlowWaitbars, 'Checked'),'on')
                        update = ['Please, wait: ', num2str(floor(slowcount/totslow*100)), '% completed. Starting time: ', begin];
                        waitbar(slowcount/totslow, slow, update);
                    end
                    slowcount = slowcount + 1;
                end
                [tau_c,W] = eval([ODE, '(''DiffEquation'', [(k-1)*block ',Output_Step,' k*block], [Uo; Vo], options)']);
                w = [w; W(1:length(W)-1, :)];
                tau_ = [tau_; tau_c(1:length(W)-1)];
                W_o = W(length(W), :);
                Uo = W_o(1:4)';
                Vo = W_o(5:8)';

                if get(handles.radiobutton_Pert,'Value') == 1
                    [xo yo] = uv2xy(Uo, Vo);       
                    [Uo Vo] = xy2uv(xo, yo);        
                    [xo yo] = uv2xy(Uo, Vo);
                elseif get(handles.radiobutton_NonPert,'Value') == 1
                    [xo yo] = zw2xy(Uo, Vo);
                    [Uo Vo] = xy2zw(xo, yo);           
                    [xo yo] = zw2xy(Uo, Vo);
                end
%---------------------- SOSTITUZIONE --------------------------------------------------------- 
%{
                Ktot = 1.001;
                while abs(Ktot-1) > 5*eps
                    Ktot = 1/2*sqrt(xo*xo')*(yo*yo'+1) + sqrt(xo*xo')*...
                    Kp(xo,yo, lambda, epsilon1, epsilon2, epsilon3, epsilon4,...
                        deg1, deg2, deg3, deg4, Hp1, Hp2, Hp3, Hp4);
                    xo = xo/Ktot;
                end           
%}
                f = @(z)1/2*sqrt(z^2*xo*xo')*(yo*yo'+1) + sqrt(z^2*xo*xo')*...
                    Kp(z*xo,yo, lambda, epsilon1, epsilon2, epsilon3, epsilon4, epsilon5,...
                        deg1, deg2, deg3, deg4, deg5, Hp1, Hp2, Hp3, Hp4, Hp5) - 1;
                a = fzero(f,1);
                xo =a*xo;

%---------------------------------------------------------------------------------------------                
                if get(handles.radiobutton_Pert,'Value') == 1
                    [Uo Vo] = xy2uv(xo, yo);
                elseif get(handles.radiobutton_NonPert,'Value') == 1
                    [Uo Vo] = xy2zw(xo, yo);
                    Uo = Uo'; Vo = Vo';
                end        

            end
            if (n_cycle*block ~= tau_total)
                [tau_c,W] = eval([ODE, '(''DiffEquation'', [n_cycle*block ',Output_Step,' tau_total], [Uo; Vo], options)']);
                W = [w; W];
                tau_c = [tau_; tau_c];
            else
                W = w;
                tau_c = tau_;
            end
%--------------------------------------------------------------------------
        else
% Original        [tau_c,W] = eval([ODE, '(''DiffEquation'', [0 ',Output_Step,'tau_total], [Uo; Vo], options)']);
            longblock = str2num(get(handles.edit_ClearMemory,'String'));
            deltaFMFT = 2^(floor(log2(str2num(get(handles.ComputeLFA,'String'))/str2num(OutStep))));
            n_cycle = floor(tau_total/longblock); totslow = floor(tau_total/longblock);
            k = 1; slowcount = 1; w = []; tau_ = [];
            for k = 1:n_cycle
                if length(w) >= longblock/str2num(OutStep) - 1;
                    if (strcmp(get(handles.LGG3_long_FMFT, 'Checked'),'on')) |...
                            (strcmp(get(handles.LSD_long_FMFT, 'Checked'),'on'))  
                        Long_Freq_Analysis
                    end
                    clear w tau_
                    w = []; tau_ = [];
                    if strcmp(get(handles.SlowWaitbars, 'Checked'),'on')
                        update = ['Please, wait: ', num2str(floor(slowcount/totslow*100)), '% completed. Starting time: ', begin];
                        waitbar(slowcount/totslow, slow, update);
                    end
                    slowcount = slowcount + 1;
                end
                [tau_c,W] = eval([ODE, '(''DiffEquation'', [(k-1)*longblock ',Output_Step,' k*longblock], [Uo; Vo], options)']);
                w = [w; W(1:length(W)-1, :)];
                tau_ = [tau_; tau_c(1:length(W)-1)];
                W_o = W(length(W), :);
                Uo = W_o(1:4)';
                Vo = W_o(5:8)';   
            end
            if (n_cycle*longblock ~= tau_total)
                [tau_c,W] = eval([ODE, '(''DiffEquation'', [n_cycle*longblock ',Output_Step,' tau_total], [Uo; Vo], options)']);
                W = [w; W];
                tau_c = [tau_; tau_c];
            else
                W = w;
                tau_c = tau_;
            end   
        end

        t_elaps = toc;
        t_elaps = num2str(t_elaps);    
        if strcmp(get(handles.waitbars, 'Checked'),'on')
            close(h)                                                %%%%%     waitbar
        end
        if strcmp(get(handles.SlowWaitbars, 'Checked'),'on')
            waitbar(1, slow, 'Finish!!!')
%       close(slow)                                             %%%%%     slow waitbar
        end     
    end
    if strcmp(get(handles.perform, 'Checked'),'on')
        if strcmp(num2str(perfid), '')
            perfid = fopen( 'Perform/Perform.txt', 'wt');
            fprintf(perfid, '*************************************************************************************************\n');
        end
        fprintf(perfid, '                             Regularization method          \n\n');
        fprintf(perfid, ['ODE solver: ',ODE,', Cycle number: ',num2str(tau_total),...
            ', Relative tolerance: ',num2str(Rel_Tol),...
            ', Integration time: ',t_elaps,' sec.\n']);
        fprintf(perfid, '*************************************************************************************************\n');
    end
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
if get(handles.radiobutton_Pert,'Value') == 1
    Calculate_Trajectories
elseif get(handles.radiobutton_NonPert,'Value') == 1
    Calculate_TrajectoriesNonPert
end
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
norma_q = sqrt((sum((q_vec.^2)'))');
sm_ax = norma_q./(2-((sum((p_vec.^2)'))').*norma_q);
if all(sm_ax > zeros(size(sm_ax)))
% do nothing
else
    if strcmp(get(handles.warningbox, 'Checked'),'on')
        warndlg('Some values of the semimajor axis are negative or null. It may be that the point is escaping to infinity.')
    end
end
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
set(handles.pushbutton_Trajectories, 'Enable', 'on')
set(handles.pushbutton_Errors, 'Enable', 'on')
set(handles.pushbutton_tauKepler, 'Enable', 'on')
set(handles.pushbutton_tKepler, 'Enable', 'on')
set(handles.pushbutton_Time, 'Enable', 'on')
set(handles.pushbutton_AA_Delaunay, 'Enable', 'on')
set(handles.pushbutton_AA_Delaunayprime, 'Enable', 'on')
set(handles.pushbutton_AA_Pauli, 'Enable', 'on')
set(handles.pushbutton_AA_Pauliprime, 'Enable', 'on')
set(handles.pushbutton_AA_User, 'Enable', 'on')
set(handles.pushbutton_UserFunction, 'Enable', 'on')
set(handles.pushbutton_tau_Step, 'Enable', 'on')
set(handles.pushbutton_Pauli, 'Enable', 'on')
if strcmp(get(handles.uipanel_Pauli,'Visible') , 'on') == 1
    set(handles.pushbutton_Pauliprime, 'Enable', 'on')
    set(handles.pushbutton_AA_Delaunayprime, 'Enable', 'on')
    set(handles.pushbutton_AA_Pauliprime, 'Enable', 'on')
else
    set(handles.pushbutton_Pauliprime, 'Enable', 'off')
    set(handles.pushbutton_AA_Delaunayprime, 'Enable', 'off')
    set(handles.pushbutton_AA_Pauliprime, 'Enable', 'off')
end

set(handles.uipanel_User, 'Visible', 'on')
set(handles.radiobutton_Pauliprime, 'Visible', 'on')
set(handles.radiobutton_Delaunay, 'Visible', 'on')
set(handles.radiobutton_Pauli, 'Visible', 'on')
set(handles.radiobutton_User, 'Visible', 'on')
set(handles.pushbutton_FFT, 'Enable', 'on')
set(handles.pushbutton_FMFT, 'Enable', 'on')
set(handles.pushbutton_wavelet, 'Enable', 'on')

if strcmp(get(handles.uipanel_Pauli,'Visible') , 'on') == 1
    set(handles.radiobutton_Delaunayprime, 'Enable', 'on')
    set(handles.radiobutton_Pauliprime, 'Enable', 'on')
else
    set(handles.radiobutton_Delaunayprime, 'Enable', 'off')
    set(handles.radiobutton_Pauliprime, 'Enable', 'off') 
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
